import 'dart:async';
import 'dart:crypto';
import 'dart:io';
import 'dart:isolate';

/**
 * Computes the SHA256 hash for the incoming data. Intercepts all events and
 * emits the computed hash as data-event when the incoming is done.
 */
class SHA256Transform extends StreamTransformer {
  SHA256 hash = new SHA256();

  void handleData(List<int> data, Sink sink) {
    hash.update(data);
  }

  void handleError(ASyncError error, Sink sink) {
    sink.add(CryptoUtils.bytesToHex(hash.digest()));
    sink.close();
  }

  void handleDone(Sink sink) {
    sink.add(CryptoUtils.bytesToHex(hash.digest()));
    sink.close();
  }
}

Stream<String> compute(String path) {
  // If the path doesn't exist, it's treated as an empty file.
  print("Serving $path");
  return new File(path).openForRead().transform(new SHA256Transform());
}

void initIsolate() {
  var sink = null;
  var subscription = stream.subscribe();
  subscription.onData((data) {
    compute(data[0]).pipe(data[1]);
  });
  subscription.onDone(() {
    sink.close();
  });
}

void main() {
  // Create a pool of 32 isolates to compute from. Plain round-robin
  // distribution of jobs
  var pool = new List(32);
  int current = 0;
  for (int i = 0; i < pool.length; i++) {
    pool[i] = streamSpawnFunction(initIsolate);
  }

  Sink getSink() {
    return pool[current++ % pool.length];
  }

  Stream<String> getHash(String path) {
    var sink = getSink();
    var box = new MessageBox();
    sink.add([path, box.sink]);
    return box.stream;
  }

  var server = new HttpServer();
  server.listen("0.0.0.0", 8080);
  server.defaultRequestHandler = (request, response) {
    Process.start('cat', []).then((process) {
      getHash(".${request.path}")
        .transform(new Utf8Encoder())
        .pipe(process);
      process.stdoutStream.pipe(response);
    });
  };
}

